"use strict";

var _interopRequireWildcard = require("@babel/runtime-corejs2/helpers/interopRequireWildcard");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Injector = void 0;

var semver = _interopRequireWildcard(require("semver"));

var _eventId = require("./eventId");

const EMIT_VERSION = "0.4.21";

class Injector {
  constructor(contract) {
    this.contract = contract;
    this.isEmitSupported = void 0;
    this.isEmitSupported = semver.gte(this.contract.solcVersion, EMIT_VERSION);
  }

  process(injectionPoint) {
    switch (injectionPoint.type) {
      case "statement":
        return this.statement(injectionPoint);

      case "contractDefinition":
        return this.contractDefinition(injectionPoint);
    }
  }

  statement(injectionPoint) {
    const data = `${this.isEmitSupported ? "emit" : ""} __StatementCoverage(${(0, _eventId.encrypt)(this.contract.id, injectionPoint.id)});`;
    this.insertAt(injectionPoint.location.start.line - 1, data);
  }

  contractDefinition(injectionPoint) {
    const data = ["event __StatementCoverage(uint32 value);"].join("\n");
    this.insertAt(injectionPoint.location.start.line, data);
  }

  insertAt(line, data) {
    const lines = this.contract.source.split("\n");
    lines.splice(line, 0, data);
    this.contract.source = lines.join("\n");
  }

}

exports.Injector = Injector;
//# sourceMappingURL=injector.js.map