"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Instrumenter = void 0;

class Instrumenter {
  constructor(contract) {
    this.contract = contract;
    this.injectionPoints = [];
  }

  getInjectionPoints() {
    return this.injectionPoints.sort((a, b) => b.location.start.line - a.location.start.line);
  }

  instrumentContractDefinition(node) {
    if (!node.loc) {
      return;
    }

    if (node.loc.start.line === node.loc.end.line) {
      return;
    }

    this.addInjectionPoints("contractDefinition", 1, node.loc);
  }

  instrumentFunction(node) {
    if (!node.loc) {
      return;
    }

    if (!node.body || !node.body.loc) {
      return;
    }

    if (node.body.loc.start.line === node.body.loc.end.line) {
      return;
    }

    this.contract.addFunction(node.loc, node.name, node.body.loc);
  }

  instrumentStatement(node) {
    if (!node.loc) {
      return;
    }

    const id = this.contract.addStatement(node.loc);
    this.addInjectionPoints("statement", id, node.loc);
  }

  instrumentIfStatement(node) {
    if (!node.loc) {
      return;
    }

    const locations = [node.trueBody, node.falseBody].reduce((acc, body) => {
      if (body && body.loc) {
        acc.push(body.loc);
      }

      return acc;
    }, []);
    this.contract.addBranch(node.loc.start.line, "if", locations);
  }

  addInjectionPoints(type, id, location) {
    this.injectionPoints.push({
      type,
      id,
      location
    });
  }

}

exports.Instrumenter = Instrumenter;
//# sourceMappingURL=instrumenter.js.map